import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
import { unwrap, isObservable } from "knockout";
import { isObservableProperty, observableProperty } from "../../../framework/observable";
import { EditorView, basicSetup } from "codemirror";
import { EditorState } from "@codemirror/state";
import { keymap } from "@codemirror/view";
import { html } from "@codemirror/lang-html";
import { indentWithTab } from "@codemirror/commands";
import { foldAll } from "@codemirror/language";
import { ButtonView } from "../../html/Button/Button";
import { prettifyHtml } from "../../../utilities/html/html";
import { DivView } from "../../html/Div/Div";
export class HtmlEditorViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (options) {
            if (isObservableProperty(options.value)) {
                this.value = options.value;
            }
            else {
                this.value = observableProperty(options.value);
            }
            if (!options.eventListeners) {
                options.eventListeners = {};
            }
        }
    }
    value;
}
export class HtmlEditorView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            viewType: "HtmlEditor",
            attributes: {
                tabindex: 1
            },
            style: {
                display: "flex",
                flexDirection: "column",
                border: "1px solid #a3d0e4"
            }
        }, designTimeOptions));
    }
    editor;
    build() {
        super.build();
        const viewModel = this.viewModel;
        const foldAllButton = new ButtonView({
            content: "Collapse all",
            style: {
                margin: "3px"
            },
            eventListeners: {
                click: {
                    listener: () => {
                        foldAll(this.editor);
                    }
                }
            }
        });
        const formatButton = new ButtonView({
            content: "Format",
            style: {
                margin: "3px"
            },
            eventListeners: {
                click: {
                    listener: () => {
                        var formattedHtml = prettifyHtml(this.editor.state.doc.toString());
                        this.editor.dispatch({ changes: { from: 0, to: this.editor.state.doc.length, insert: formattedHtml } });
                    }
                }
            }
        });
        const saveButton = new ButtonView({
            content: "Save",
            style: {
                margin: "3px"
            },
            eventListeners: {
                click: {
                    listener: () => {
                        if (isObservableProperty(viewModel.value)) {
                            viewModel.value.set(this.editor.state.doc.toString());
                        }
                    }
                }
            }
        });
        const buttons = new DivView({
            style: {
                display: "flex"
            },
            children: [
                foldAllButton,
                formatButton,
                saveButton
            ]
        });
        this.element.appendChild(buttons.render());
        const state = EditorState.create({
            extensions: [
                basicSetup,
                html(),
                EditorView.lineWrapping,
                keymap.of([indentWithTab])
            ]
        });
        this.editor = new EditorView({
            state: state,
            parent: this.element,
        });
        this.editor.dom.style.overflow = "auto";
        this.editor.dispatch({ changes: { from: 0, to: this.editor.state.doc.length, insert: unwrap(viewModel.value) } });
        if (isObservable(viewModel.value)) {
            viewModel.value.subscribe((html) => {
                this.editor.dispatch({ changes: { from: 0, to: this.editor.state.doc.length, insert: html } });
            });
        }
    }
}
registerView(HtmlEditorViewModel, HtmlEditorView);
//# sourceMappingURL=HtmlEditor.js.map