import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
import { unwrap, isObservable } from "knockout";
import { JSONFormatter } from "json-viewer-awesome/lib/json-formatter";
export class JsonFormatterViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (!options) {
            options = {};
        }
        this.json = options.json;
        this.showArrayIndex = !(options.showArrayIndex === false);
        this.quotesOnKeys = !(options.quotesOnKeys === false);
        this.displayDataTypes = !(options.displayDataTypes === false);
        this.displayObjectSize = !(options.displayObjectSize === false);
        this.sortKeys = !!options.sortKeys;
        this.collapsed = !(options.collapsed === false);
    }
    json;
    showArrayIndex;
    quotesOnKeys;
    displayDataTypes;
    displayObjectSize;
    sortKeys;
    collapsed;
}
export class JsonFormatterView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            viewType: "JsonFormatter"
        }, designTimeOptions));
    }
    build() {
        super.build();
        const viewModel = this.viewModel;
        this.setJson(unwrap(viewModel.json));
        if (isObservable(viewModel.json)) {
            viewModel.json.subscribe((json) => {
                this.setJson(json);
            });
        }
    }
    setJson(json) {
        if (json) {
            if (typeof json === "object") {
                const jsonFormatter = new JSONFormatter({
                    showArrayIndex: this.viewModel.showArrayIndex,
                    quotesOnKeys: this.viewModel.quotesOnKeys,
                    displayDataTypes: this.viewModel.displayDataTypes,
                    displayObjectSize: this.viewModel.displayObjectSize,
                    sortKeys: false,
                    collapsed: true,
                    iconStyle: 0,
                    theme: undefined,
                    src: json
                });
                this.element.replaceChildren(jsonFormatter.render());
            }
            else {
                this.element.innerHTML = json.toString();
            }
        }
        else {
            this.empty();
        }
    }
}
registerView(JsonFormatterViewModel, JsonFormatterView);
//# sourceMappingURL=JsonFormatter.js.map