import * as kendo from "@progress/kendo-ui/js/kendo.editor.js";
import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
import { unwrap, isObservable, computed, observable } from "knockout";
import { isObservableProperty, observableProperty } from "../../../framework/observable";
import { DivView } from "../../html/Div/Div";
import { ButtonView } from "../../html/Button/Button";
import { prettifyHtml } from "../../../utilities/html/html";
export class EditorViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (options) {
            if (isObservableProperty(options.value)) {
                this.value = options.value;
            }
            else {
                this.value = observableProperty(options.value);
            }
            if (!options.eventListeners) {
                options.eventListeners = {};
            }
            this.tools = options.tools;
        }
    }
    value;
    tools;
}
export class EditorView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            viewType: "KendoEditor",
            style: {
                display: "flex",
                flexDirection: "column"
            }
        }, designTimeOptions));
    }
    editor;
    build() {
        super.build();
        const viewModel = this.viewModel;
        const htmlMode = observable(false);
        const divHeader = new DivView({
            children: [
                new ButtonView({
                    content: computed(() => {
                        return htmlMode() ? "wysiwyg" : "view html source";
                    }),
                    eventListeners: {
                        click: {
                            listener: () => {
                                htmlMode(!htmlMode());
                            }
                        }
                    },
                    style: {
                        width: "120px",
                        marginTop: "2px"
                    }
                })
            ]
        }).render();
        const divElement = new DivView({
            visible: computed(() => {
                return !htmlMode();
            }),
            style: {
                flexGrow: 1
            }
        }).render();
        const textAreaElement = new View({
            tagName: "textarea",
            viewModel: {
                visible: htmlMode,
                style: {
                    flexGrow: 1,
                    borderColor: "transparent",
                    borderRadius: "4px"
                }
            }
        }).render();
        this.element.appendChild(divElement);
        this.element.appendChild(textAreaElement);
        this.element.appendChild(divHeader);
        const html = unwrap(viewModel.value);
        divElement.innerHTML = html;
        textAreaElement.value = prettifyHtml(html);
        if (isObservable(viewModel.value)) {
            viewModel.value.subscribe((value) => {
                divElement.innerHTML = value;
                textAreaElement.value = value;
            });
        }
        this.editor = new kendo.ui.Editor(divElement, {
            tools: viewModel.tools
        });
        this.editor.bind("change", () => {
            if (isObservableProperty(viewModel.value)) {
                viewModel.value.set(divElement.innerHTML);
            }
        });
        textAreaElement.addEventListener("change", () => {
            if (isObservableProperty(viewModel.value)) {
                viewModel.value.set(textAreaElement.value);
            }
        });
    }
}
registerView(EditorViewModel, EditorView);
//# sourceMappingURL=Editor.js.map