import * as kendo from "@progress/kendo-ui/js/kendo.grid.js";
import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
import { unwrap, isObservable, utils } from "knockout";
import { knockoutObservableToKendoObservableArray, getUnderlyingObject } from "../ObservableObject/ObservableObject";
export class GridViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (options) {
            this.items = options.items;
            this.idProperty = options.idProperty;
            this.columns = options.columns;
            this.pageSize = options.pageSize;
            this.sortable = options.sortable;
            this.scrollable = options.scrollable;
            this.editable = options.editable;
            this.filterable = options.filterable;
            this.toolbar = options.toolbar;
            this.selectable = options.selectable;
            this.selectedItem = options.selectedItem;
            this.selectedItems = options.selectedItems;
            this.onDataSourceChange = options.onDataSourceChange;
            this.onDataSourceAdd = options.onDataSourceAdd;
            this.onDataSourceRemove = options.onDataSourceRemove;
        }
    }
    items;
    idProperty;
    columns;
    pageSize;
    sortable;
    scrollable;
    editable;
    filterable;
    toolbar;
    selectable;
    selectedItem;
    selectedItems;
    onDataSourceChange;
    onDataSourceAdd;
    onDataSourceRemove;
    selectItem(index = null) {
        if (!index) {
            index = 0;
        }
        const items = unwrap(this.items);
        const item = items[index];
        if (item) {
            if (this.selectedItem) {
                const observableProperty = this.selectedItem;
                if (observableProperty.set) {
                    observableProperty.set(item);
                }
            }
            if (this.selectedItems) {
                const observableArrayProperty = this.selectedItems;
                if (observableArrayProperty.set) {
                    observableArrayProperty.set([item]);
                }
            }
        }
    }
}
export class GridView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({ viewType: "KendoGrid" }, designTimeOptions));
    }
    grid;
    dataSource;
    getItemById(id) {
        if (this.viewModel.idProperty) {
            const items = this.dataSource.data();
            return utils.arrayFirst(items, (item) => { return item[this.viewModel.idProperty] === id; });
        }
        throw "Can only getItem when idProperty is set";
    }
    build() {
        super.build();
        const viewModel = this.viewModel;
        this.dataSource = new kendo.data.DataSource({
            data: knockoutObservableToKendoObservableArray(viewModel.items),
            pageSize: this.viewModel.pageSize || 30,
            change: (event) => {
                var items = event.items;
                switch (event.action) {
                    case "itemchange":
                        if (viewModel.onDataSourceChange) {
                            viewModel.onDataSourceChange(items, event.sender);
                        }
                        break;
                    case "add":
                        if (viewModel.onDataSourceAdd) {
                            viewModel.onDataSourceAdd(items, event.sender);
                        }
                        break;
                    case "remove":
                        if (viewModel.onDataSourceRemove) {
                            viewModel.onDataSourceRemove(items, event.sender);
                        }
                        break;
                }
            }
        });
        this.grid = new kendo.ui.Grid(this.element, {
            dataSource: this.dataSource,
            columns: this.viewModel.columns,
            toolbar: this.viewModel.toolbar,
            scrollable: this.viewModel.scrollable,
            sortable: this.viewModel.sortable,
            editable: this.viewModel.editable,
            filterable: this.viewModel.filterable,
            change: (event) => {
                if (this.viewModel.selectedItem) {
                    const selectedItem = getSelectedDataItem(event.sender);
                    const observableProperty = this.viewModel.selectedItem;
                    if (observableProperty.set) {
                        observableProperty.set(selectedItem);
                    }
                }
                if (this.viewModel.selectedItems) {
                    const selectedItems = getSelectedDataItems(event.sender);
                    const observableArrayProperty = this.viewModel.selectedItems;
                    if (observableArrayProperty.set) {
                        observableArrayProperty.set(selectedItems);
                    }
                }
                if (this.viewModel.onGridChange) {
                    this.viewModel.onGridChange(getSelectedDataItems(event.sender), event.sender);
                }
            },
            selectable: this.viewModel.selectable
        });
        if (viewModel.onRowContextMenu) {
            this.grid.table.contextmenu((event) => {
                event.preventDefault();
                var target = event.target;
                var row;
                if (target instanceof HTMLElement) {
                    switch (target.tagName.toLowerCase()) {
                        case "td":
                            row = target.parentElement;
                            break;
                        case "tr":
                            row = target;
                            break;
                    }
                }
                if (row) {
                    let dataItem = this.grid.dataItem(row);
                    if (dataItem) {
                        const underlyingObject = getUnderlyingObject(dataItem);
                        if (underlyingObject) {
                            dataItem = underlyingObject;
                        }
                        setTimeout(() => {
                            viewModel.onRowContextMenu(event, dataItem);
                        });
                    }
                    ;
                }
            });
        }
        if (isObservable(viewModel.items)) {
            viewModel.items.subscribe((items) => {
                if (items) {
                    this.dataSource.data(new kendo.data.ObservableArray(knockoutObservableToKendoObservableArray(items)));
                }
                else {
                    this.dataSource.data([]);
                }
            });
        }
        if (isObservable(viewModel.selectedItem)) {
            viewModel.selectedItem.subscribe((selectedItem) => {
                if (selectedItem && viewModel.idProperty) {
                    setTimeout(() => {
                        const id = selectedItem[viewModel.idProperty];
                        const item = this.getItemById(id);
                        this.grid.tbody.find("tr.k-state-selected").removeClass("k-state-selected");
                        this.grid.tbody.find("tr[data-uid='" + item.uid + "']").addClass("k-state-selected");
                    }, 10);
                }
                else {
                    this.grid.select();
                }
            });
        }
        if (isObservable(viewModel.selectedItems)) {
            viewModel.selectedItems.subscribe((selectedItems) => {
                if (selectedItems && selectedItems.length > 0 && viewModel.idProperty) {
                    setTimeout(() => {
                        this.grid.tbody.find("tr.k-state-selected").removeClass("k-state-selected");
                        for (let index = 0; index < selectedItems.length; index++) {
                            const selectedItem = selectedItems[index];
                            const id = selectedItem[viewModel.idProperty];
                            const item = this.getItemById(id);
                            this.grid.tbody.find("tr[data-uid='" + item.uid + "']").addClass("k-state-selected");
                        }
                    }, 10);
                }
                else {
                    this.grid.select();
                }
            });
        }
        setTimeout(() => {
            this.grid.resize(true);
        }, 5);
    }
}
export function getSelectedDataItem(grid) {
    const selection = grid.select();
    if (selection.length === 1) {
        const dataItem = grid.dataItem(selection[0]);
        const underlyingObject = getUnderlyingObject(dataItem);
        if (underlyingObject) {
            return underlyingObject;
        }
        return dataItem.toJSON();
    }
    return null;
}
export function getSelectedDataItems(grid) {
    const selection = grid.select();
    const selectedDataItems = new Array();
    if (selection && selection.length > 0) {
        for (let index = 0; index < selection.length; index++) {
            const dataItem = grid.dataItem(selection[index]);
            const underlyingObject = getUnderlyingObject(dataItem);
            if (underlyingObject) {
                selectedDataItems.push(underlyingObject);
            }
            else {
                selectedDataItems.push(dataItem.toJSON());
            }
        }
    }
    return selectedDataItems;
}
registerView(GridViewModel, GridView);
//# sourceMappingURL=Grid.js.map