import { isObservable, unwrap } from "knockout";
import * as leaflet from "leaflet";
import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel } from "../../../framework/ViewModel";
export class MapViewModel extends ViewModel {
    constructor(options) {
        super(options);
        if (options) {
            this.renderer = options.renderer;
            this.center = options.center;
            this.zoom = options.zoom;
            this.tileLayers = options.tileLayers;
            this.layers = options.layers;
            this.controls = options.controls;
            this.zoomInTitle = options.zoomInTitle;
            this.zoomOutTitle = options.zoomOutTitle;
        }
    }
    renderer;
    center;
    zoom;
    south;
    north;
    west;
    east;
    tileLayers;
    layers;
    controls;
    zoomInTitle;
    zoomOutTitle;
}
export class MapView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "div",
            viewModel: viewModel
        }, extendViewModel({
            viewType: "LeafletMap"
        }, designTimeOptions));
    }
    map;
    layers = new Array();
    controls = new Array();
    build() {
        super.build();
        const viewModel = this.viewModel;
        if (!viewModel.center) {
            viewModel.center = burtonville;
        }
        const options = {
            renderer: viewModel.renderer,
            zoomControl: false
        };
        setTimeout(() => {
            this.map = leaflet.map(this.element, options).setView(unwrap(viewModel.center), this.viewModel.zoom);
            leaflet.control.zoom({
                position: "topleft",
                zoomInTitle: viewModel.zoomInTitle,
                zoomOutTitle: viewModel.zoomOutTitle
            }).addTo(this.map);
            if (this.viewModel.eventListeners) {
                Object.keys(this.viewModel.eventListeners).forEach((key) => {
                    this.map.on(key, (event) => {
                        this.viewModel.eventListeners[key](event);
                    });
                });
            }
            this.setTileLayers(viewModel.tileLayers);
            this.setLayers(unwrap(viewModel.layers));
            if (isObservable(viewModel.layers)) {
                viewModel.layers.subscribe((layers) => {
                    this.setLayers(layers);
                });
            }
            this.setControls(unwrap(viewModel.controls));
            if (isObservable(viewModel.controls)) {
                viewModel.controls.subscribe((controls) => {
                    this.setControls(controls);
                });
            }
            if (isObservable(viewModel.center)) {
                viewModel.center.subscribe((center) => {
                    this.map.panTo(center);
                });
            }
        }, 100);
    }
    setTileLayers(tileLayers) {
        if (tileLayers) {
            tileLayers.forEach((tileLayer) => {
                tileLayer.addTo(this.map);
            });
        }
    }
    setLayers(layers) {
        this.layers.forEach((layer) => {
            this.map.removeLayer(layer);
        });
        this.layers = [];
        if (layers) {
            layers.forEach((layer) => {
                this.layers.push(layer);
                layer.addTo(this.map);
            });
        }
    }
    setControls(controls) {
        this.controls.forEach((control) => {
            this.map.removeControl(control);
        });
        this.controls = [];
        if (controls) {
            controls.forEach((control) => {
                this.controls.push(control);
                control.addTo(this.map);
            });
        }
    }
}
registerView(MapViewModel, MapView);
export const burtonville = [50.283939, 5.964070];
//# sourceMappingURL=Map.js.map