export declare const mouseEvents: {
    click: boolean;
    contextmenu: boolean;
    mousedown: boolean;
    mouseenter: boolean;
    mouseleave: boolean;
    mousemove: boolean;
    mouseout: boolean;
    mouseover: boolean;
    mouseup: boolean;
};
export declare const touchEvents: {
    touchcancel: boolean;
    touchend: boolean;
    touchmove: boolean;
    touchstart: boolean;
    longtap: boolean;
    swipeleft: boolean;
    swiperight: boolean;
    swipeup: boolean;
    swipedown: boolean;
};
export type MouseE = "click" | "contextmenu";
export declare function isGlobalEvent(event: keyof UiElementEventMap): boolean;
export interface UiElementEventMap extends HTMLElementEventMap {
    "tap": Event;
    "dbltap": Event;
    "longtap": Event;
    "swipeleft": Event;
    "swiperight": Event;
    "swipeup": Event;
    "swipedown": Event;
    "textInput": Event;
}
export interface IMouseEventListener extends IEventListener {
    listener: (event: MouseEvent) => void;
}
export interface IFocusEventListener extends IEventListener {
    listener: (event: FocusEvent) => void;
}
export interface IContextMenuEventListener extends IEventListener {
    listener: (event: PointerEvent) => void;
}
export interface IGenericEventListener extends IEventListener {
    listener: (event: Event) => void;
}
export interface IEventListener {
    options?: IEventListenerOptions;
}
export interface MouseEventListeners {
    click?: IMouseEventListener;
}
export interface FocusEventListeners {
    focusout?: IFocusEventListener;
}
export interface ContextMenuEventListeners {
    contextMenu?: IContextMenuEventListener;
}
export interface EventListeners extends MouseEventListeners, FocusEventListeners, ContextMenuEventListeners {
    change?: IGenericEventListener;
    load?: IGenericEventListener;
}
type EventKey = "Enter" | "ArrowDown" | "ArrowUp" | "ArrowLeft" | "ArrowRight";
export interface IEventListenerOptions extends AddEventListenerOptions {
    eventKey?: EventKey;
    altKey?: boolean;
    shiftKey?: boolean;
    ctrlKey?: boolean;
    mouseLeft?: boolean;
    mouseMiddle?: boolean;
    mouseRight?: boolean;
}
export {};
