import { isObservable, observable, observableArray, pureComputed } from "knockout";
export function isObservableProperty(instance) {
    return instance && !!instance.set;
}
export function observableProperty(value, setter, notifyAlways = false) {
    let internalProperty;
    if (value === undefined) {
        value = null;
    }
    if (isObservable(value)) {
        internalProperty = value;
    }
    else {
        internalProperty = observable(value);
    }
    if (notifyAlways) {
        internalProperty.extend({ notify: "always" });
    }
    const computed = pureComputed({ read: internalProperty }).extend({ notify: "always" });
    computed.set = (newValue, triggerSetter = true) => {
        if (setter && triggerSetter) {
            setter(newValue, internalProperty);
        }
        else {
            internalProperty(newValue);
        }
    };
    return computed;
}
export function observableArrayProperty(items, setter) {
    let internalProperty;
    if (!items) {
        items = [];
    }
    if (isObservable(items)) {
        internalProperty = items;
    }
    else {
        internalProperty = observableArray(items);
    }
    const computed = pureComputed({ read: internalProperty }).extend({ notify: "always" });
    computed.set = (value, triggerSetter = true) => {
        if (setter && triggerSetter) {
            setter(value, internalProperty);
        }
        else {
            internalProperty(value);
        }
    };
    computed.push = (...itemsToPush) => {
        internalProperty.push(...itemsToPush);
        return itemsToPush.length;
    };
    computed.map = (callBack, thisArg) => {
        return internalProperty().map(callBack, thisArg);
    };
    return computed;
}
//# sourceMappingURL=observable.js.map