import { newGuid } from "../uniqueId/uniqueId";
var testAccessToken = null;
export function setTestAccessToken(accessToken) {
    testAccessToken = accessToken;
}
var isTestRun = false;
export function setIsTestRun(testRun) {
    isTestRun = testRun;
}
var testUserId;
export function setTestUserId(userId) {
    testUserId = userId;
}
var tokenProvider;
var additionalHeaders;
export function initialize(authorizationTokenProvider, headersProvider) {
    tokenProvider = authorizationTokenProvider;
    additionalHeaders = headersProvider;
}
export async function get(input, authorized, responseType, init) {
    if (!init) {
        init = {};
    }
    await setGetInitValues(init, authorized, responseType);
    const response = await fetch(input, init);
    return await getResponseValue(response, responseType);
}
export async function post(input, data, authorized, responseType, init) {
    if (!init) {
        init = {};
    }
    await setPostInitValues(init, authorized, responseType, data);
    const response = await fetch(input, init);
    return await getResponseValue(response, responseType);
}
function getFilename(response) {
    const contentDispositionHeader = response.headers.get("Content-Disposition");
    if (contentDispositionHeader) {
        const parts = contentDispositionHeader.split(";");
        for (let part of parts) {
            if (part.indexOf("filename=") > -1) {
                return part.replace("filename=", "").trim();
            }
        }
    }
    return null;
}
async function getResponseValue(response, responseType) {
    if (response.ok) {
        switch (responseType) {
            case "void":
                return null;
            case "json":
                return response.json();
            case "blob":
                return {
                    blob: await response.blob(),
                    name: getFilename(response)
                };
            case "arrayBuffer":
                return response.arrayBuffer();
            default:
                throw new Error("The responseType" + responseType + " is not valid");
        }
    }
    const errorJson = await response.json();
    throw new Error(errorJson.message);
}
async function setGetInitValues(init, authorized, responseType) {
    await setHeaders(init, authorized, responseType);
}
async function setPostInitValues(init, authorized, responseType, data) {
    await setHeaders(init, authorized, responseType);
    if (!init.method) {
        init.method = "POST";
    }
    if (!init.mode) {
        init.mode = "cors";
    }
    if (!init.cache) {
        init.cache = "no-cache";
    }
    if (!init.credentials) {
        init.credentials = "same-origin";
    }
    if (!init.headers) {
        init.headers = {};
    }
    if (!init.headers["Content-Type"] && !(data instanceof FormData)) {
        init.headers["Content-Type"] = "application/json";
    }
    if (data instanceof FormData) {
        delete init.headers["Content-Type"];
    }
    if (!init.redirect) {
        init.redirect = "follow";
    }
    if (!init.referrerPolicy) {
        init.referrerPolicy = "no-referrer";
    }
    if (!init.body) {
        if (data instanceof FormData) {
            init.body = data;
        }
        else {
            init.body = JSON.stringify(data);
        }
    }
}
async function setHeaders(init, authorized, responsetype) {
    if (!init.headers) {
        init.headers = {};
    }
    if (responsetype === "json") {
        init.headers["Accept"] = "application/json";
    }
    if (isTestRun) {
        init.headers["TestUserId"] = testUserId;
    }
    init.headers["X-Request-ID"] = newGuid();
    if (additionalHeaders) {
        additionalHeaders(init.headers);
    }
    if (testAccessToken) {
        init.headers["Authorization"] = "Bearer " + testAccessToken;
    }
    else {
        if (authorized) {
            if (tokenProvider) {
                init.headers["Authorization"] = await tokenProvider();
            }
            else {
                throw "Token provider not set";
            }
        }
    }
}
//# sourceMappingURL=fetch.js.map