import { getStateFromQueryString, stateToQueryString } from "../querystring/querystring";
export class Router {
    constructor(options) {
        this.options = options;
        if (options.routes) {
            this.setRoutes(options.routes);
        }
    }
    routeDictionary = {};
    popstateHandler(event) {
        const pathname = window.location.pathname;
        const state = event.state || getStateFromQueryString();
        const route = this.routeDictionary[pathname];
        if (route) {
            this.options.change(route, state);
        }
        else {
            this.options.routeMissing(pathname, state);
        }
    }
    options;
    start() {
        window.addEventListener("popstate", (event) => { this.popstateHandler(event); });
        window.dispatchEvent(new Event("popstate"));
    }
    setRoutes(routes, replace = false) {
        if (replace) {
            this.routeDictionary = {};
        }
        if (routes) {
            routes.forEach((route) => {
                this.addRoute(route);
            });
        }
    }
    destroy() {
        window.removeEventListener("popstate", this.popstateHandler);
    }
    addRoute(route) {
        let url = route.url;
        if (this.options.root) {
            url = this.options.root + url;
        }
        if (url[0] !== "/") {
            url = "/" + url;
        }
        this.routeDictionary[url] = route;
    }
    navigate(route, state, addStateToQueryString = true, triggerChange = true) {
        let url = route;
        if (state && addStateToQueryString) {
            url = url + stateToQueryString(state);
        }
        if (!state) {
            state = {};
        }
        history.pushState(state, "", url);
        if (triggerChange) {
            window.dispatchEvent(new PopStateEvent("popstate", { state: state }));
        }
    }
}
//# sourceMappingURL=router.js.map